// Datei: pict6963.c
//
// Serielles Display mit T6963 Controller und
// sechs Eingabetastern. Das Display wird nur im
// Textmodus angesprochen !
//
// Mein Display ist ein 240x64 Display von Winstar.
// Stellt man den 6x8 Font ein hat man ein Textdisplay mit
// 8 Zeilen a 40 Zeichen !
//
// Man kann auch kleinere Displays nehmen.
// Dazu mu man nur Hhe und Breite in T6963.h ndern und mit
// CC5X neu bersetzen.
// Anzahl Zeichen pro Zeile = Breite in Pixeln / 6
// Anzahl Zeilen = Hhe / 8
//
// Die Steuerung erfolgt ber eine (noch nicht komplette)
// Emulation von Matrix Orbital Displays.
// Damit arbeitet es bei mir wunderbar mit dem LCD-Plugin
// von Winamp zusammen.
//
// Baudrate (fest) 19200, 8Bit, 1 Stopbit, keine Paritt
//
// Fr den Spektrum-Analyzer werden bereits benutzerdefinierte Zeichen
// vom PIC definiert ! Man braucht sie im Plugin also nicht einstellen !
// Wer eigene benutzerdefinierte Zeichen auerhalb des Analyzers nutzen
// mchte mu auf den Analyzer GANZ verzichten ! Oder besser gesagt das
// sieht sonst ziemlich wild aus. Sie benutzen nhmlich beide die
// benutzerdefinierbaren Zeichen von 0-7. Schade das das Plugin nicht mehr
// erlaubt :( Im T6963 sind die Zeichen von 0x89-0xFF nhmlich noch unbelegt.
//
// Der Analyzer startet bei mir nicht automatisch. Auch nicht wenn ich
// eine Taste dafr definiere und drcke. Ist das normal (Bug) ?
// Jedesmal ab in die Prefs und manuell starten !
//
// ->irgend so ein komisches Zeichen was ein wenig hnlich ist.
// Sollte ich vieleicht auch selbst definieren ;)
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 27.06.2002
// Compiler CC5x

#include "prozess.h"      // Der verwendete Prozessor
#include "pict6963.h"
#include "t6963.h"
#include "protos.h"

#include "int.c"
#include "t6963.c"
#include "serial.c"

unsigned char keys;   //Die sechs Tasten
unsigned char cgr[9]; //Puffer zum Empfang selbstdefinierter Zeichen

//*******************************************************
// Hauptprogramm
//*******************************************************
void main()
{
 unsigned char i,ch,row,col,by;
 
 CLRWDT;
 //Takt des PIC 20MHz, ein Zyklus = 200ns
 OPTION=0b11000001; //Pullups off,Prescaler auf TMR0, 1:4

 ADCON0=0;          //AD-Wandler aus
 ADCON1=0x06;       //Alle Analogeingnge auf digital

 PORTB=0b00000000;  
 PORTA=0b00000000;     
 PORTC=0b00111010;

 TRISB=0b11111111;  //0=Ausgang, 1=Eingang
 TRISA=0b11111111;  //0=Ausgang, 1=Eingang
 TRISC=0b10011000;  //

//I2C-Bus Modul einschalten
// SSPCON=0b00111000; //I2C-Master Mode
 SSPCON=0b00110110; //I2C-Slave Mode, 7Bit Adresse
 SSPADD=0x60;       //I2C Adresse des Pic setzen auf die er horchen soll
 SSPSTAT=0b10000000;  
 SSPCON2=0b00000000;  

 T1CON=0b00100001;  //Prescaler TMR1 auf 1:4, ergibt Int alle 50ms.
 TMR1IE=1;          //Timer1 Int an fr Tastenabfrage

 PEIE=1;            //Periphere Ints an
 SSPIE=1;           //I2C-Interrupt an
 GIE=1;             //Interrupts erlauben

 InitDisplay(); //Display in Textmodus
 InitUSART();

 keys=0;
 icnt=0;
 
 while(1) // Programm-Endlosschleife
  {
//################################################################
// Infrarot Empfang auf I2C-Bus abfragen
//################################################################
   if(icnt>2) //InfrarotCode empfangen ?
    {
     Transmit(idat1); //1. Byte senden
     Transmit(idat2); //2. Byte senden

     Transmit(0);     //Dummy Bytes senden zum auffllen auf 6 Bytes
     Transmit(0);
     Transmit(0);
     Transmit(0);
     
     icnt=0;       //Infrarot Empfang zurcksetzen
    }
//################################################################
// Ende Infrarot Empfang auf I2C-Bus abfragen
//################################################################


//################################################################
// Tastatur abfragen
//################################################################
// Tastaturmeldungen auf demselben COM-Port wie das Display
// Die Wiederholrate mit 50ms ist etwas zu schnell.
   if(keys>0) //Taste gedrckt ?
    {
     switch(keys)
      {
       case KEY0: by='0'; break;
       case KEY1: by='1'; break;
       case KEY2: by='2'; break;
       case KEY3: by='3'; break;
       case KEY4: by='4'; break;
       case KEY5: by='5'; break;
       default :  by='?'; break; //Den dicken Finger auf zwei Tasten gleichzeitig gehabt ;)
      }
     Transmit(by);  //Nummer des Keys
     keys=0;       //Letzten Tastendruck lschen
    }
//################################################################
// Ende Tastatur abfragen
//################################################################

//################################################################
// Serielle Schnittstelle abfragen
//################################################################
    if(rcnt>0) //Daten im Puffer ?
     {
      ch=Receive();
      if(ch==0xFE)  //Kommando Prefix von MatrixOrbital Displays
       {
newcommand:
        ch=Receive();
        if(ch==0xFE) goto newcommand;
//Viele Befehle werden hier nur abgefangen bzw. deren Parameter
//abgeholt damit sie nicht als Text ausgegeben werden ! 
        switch(ch)
         {
          case 'X': ClearScreen(); break; //Clear Display
          case 'H': LCDPos(1,1); break; //Go To Top Left
          case 'G': //Go To Position, Col, Row
             col=Receive();
             if(col==0xFE) goto newcommand;
             row=Receive();
             if(row==0xFE) goto newcommand;
             LCDPos(row,col);
          break;
          case 'N': //Create Custom Character, 1 Byte Zeichennummer (0-7), 8 Databytes
            for(i=0; i<9; i++)
             {
              ch=Receive();
              if(ch==0xFE) goto newcommand; //Kommandoprefix gefunden
              cgr[i]=ch;
             } 

             if(cgr[0]<=8) MakeCGRam();
          break;
          case 'P':  //Contrast, Parameter abholen
             ch=Receive();
             if(ch==0xFE) goto newcommand;
           break;
          case 'B':  //Backlight On, Parameter X abholen
             ch=Receive();
             if(ch==0xFE) goto newcommand;
           break;
                                         //0=immer an, 1-100 = an fr X Minuten
//          case 'F': break; //Backlight Off
//          case 'J': break; //Cursor On
//          case 'K': break; //Cursor Off
//          case 'L': break; //Cursor Left
//          case 'M': break; //Cursor Right
//          case 'S': break; //Cursor Blink On
//          case 'T': break; //Cursor Blink Off
//          case 'h': break; //Initialize Horizontal Bar Graph
//          case '|': break; //Make Horizontal Bar Graph, Spalte, Reihe, Richtung 0=R, Lnge 0-200 
//          case 'v': break; //Initialize Thick Vertical Bar Graph
//          case 's': break; //Initialize Thin Bar Vertical Graph
//          case '=': break; //Make Vertical Bar Graph, Spalte, Lnge
//          case 'C': break; //Auto Line Wrap On
//          case 'D': break; //Auto Line Wrap Off
//          case 'Q': break; //Auto Scroll On
//          case 'R': break; //Auto Scroll Off
//          case 'V': ch=Receive(); break; //General Purpose Output On, Parameter abholen
//          case 'W': ch=Receive(); break; //General Purpose Output Off, Parameter abholen
          default: break;
         }
       }
      else LCDChar(ch); //Alles andere ist Text

     }//if rcnt>0
    
//################################################################
// Ende serielle Schnittstelle abfragen
//################################################################
  } //while(1) Endlosschleife

}
